'use strict';

var _execa = _interopRequireDefault(require("execa"));

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _nodeBanner = _interopRequireDefault(require("node-banner"));

var _messages = require("../../utils/messages");

var _validate = require("../../utils/validate");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

/**
 * Returns the respective file content as an array
 *
 * @param {String} configFile - File whose content is to be read
 * @returns {String[]}
 */
var getFileContent = function getFileContent(configFile) {
  // Holds reference to the path where docker-config files reside
  var dockerConfigTemplatePath = _path["default"].join(__dirname, '..', '..', 'templates', 'docker');

  return _fs["default"].readFileSync(_path["default"].join(dockerConfigTemplatePath, configFile)).toString().split('\n');
};
/**
 * Make data directory for mongo container to mount in order to persist
 * the database.
 *
 * @returns {void}
 */


var makeDataDir = function makeDataDir() {
  _fs["default"].mkdirSync(_path["default"].join('tmp', 'data'), {
    recursive: true
  });

  var gitIgnoreContents = _fs["default"].readFileSync('.gitignore');

  var mongoGitIgnoreHeader = '# MEVN_GENERATED:MONGO';

  if (!new RegExp(mongoGitIgnoreHeader, 'g').test(gitIgnoreContents)) {
    gitIgnoreContents += ['\n', '# MEVN_GENERATED:MONGO', '/tmp', '\n'].join('\n');
  } // Write back the updated contents to .gitignore


  _fs["default"].writeFileSync('.gitignore', gitIgnoreContents);
};
/**
 * Launch multiple containers with docker-compose (client, server and mongo client)
 *
 * @returns {Promise<void>}
 */


var dockerize = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
    var dockerComposeTemplate, dockerFileTemplate, clientDockerFilePath, serverDockerFilePath, serverDockerIgnorePath, clientDockerIgnorePath;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return (0, _nodeBanner["default"])('MEVN CLI', 'Light speed setup for MEVN stack based apps.');

          case 2:
            (0, _messages.checkIfConfigFileExists)();
            _context.next = 5;
            return (0, _validate.validateInstallation)('docker');

          case 5:
            // Get the respective file contents
            dockerComposeTemplate = getFileContent('docker-compose.yml');
            dockerFileTemplate = getFileContent('Dockerfile'); // Create Dockerfile for client directory

            clientDockerFilePath = _path["default"].join('client', 'Dockerfile');

            if (!_fs["default"].existsSync(clientDockerFilePath)) {
              _fs["default"].writeFileSync(clientDockerFilePath, dockerFileTemplate.join('\n'));
            }

            if (_fs["default"].existsSync('./server')) {
              // Create Dockerfile for the server directory
              dockerFileTemplate.splice(8, 0, 'RUN npm install -g nodemon');
              dockerFileTemplate.splice(9, 0, ''); // Create Dockerfile for server directory

              serverDockerFilePath = _path["default"].join('server', 'Dockerfile');

              if (!_fs["default"].existsSync(serverDockerFilePath)) {
                _fs["default"].writeFileSync(serverDockerFilePath, dockerFileTemplate.join('\n'));
              } // Create .dockerignore within server directory


              serverDockerIgnorePath = _path["default"].join('server', '.dockerignore');

              if (!_fs["default"].existsSync(serverDockerIgnorePath)) {
                _fs["default"].writeFileSync(serverDockerIgnorePath, 'node_modules');
              } // Configure env variables for CRUD boilerplate template
              // to be consumed within the Docker container


              if (_fs["default"].existsSync(_path["default"].join('server', 'models'))) {
                if (!_fs["default"].existsSync(_path["default"].join('tmp', 'data'))) {
                  makeDataDir();
                  dockerComposeTemplate.splice(14, 0, "".concat(' '.repeat(4), "environment:"));
                  dockerComposeTemplate.splice(15, 0, "".concat(' '.repeat(6), "- DB_URL=mongodb://mongo:27017"));

                  _fs["default"].writeFileSync('docker-compose.yml', dockerComposeTemplate.join('\n'));
                }
              } else {
                dockerComposeTemplate = dockerComposeTemplate.slice(0, 19);
              }
            } // Create .dockerignore within client directory


            clientDockerIgnorePath = _path["default"].join('client', '.dockerignore');

            if (!_fs["default"].existsSync(clientDockerIgnorePath)) {
              _fs["default"].writeFileSync(clientDockerIgnorePath, 'node_modules\ndist');
            } // docker-compose.yml should only include the necessary instructions
            // for the client side


            if (!_fs["default"].existsSync('./server')) {
              dockerComposeTemplate = dockerComposeTemplate.slice(0, 10);
            } // Create docker-compose.yml at project root


            if (!_fs["default"].existsSync('docker-compose.yml')) {
              _fs["default"].writeFileSync('docker-compose.yml', dockerComposeTemplate.join('\n'));
            }

            _context.prev = 14;
            _context.next = 17;
            return _execa["default"].shell('docker-compose up', {
              stdio: 'inherit'
            });

          case 17:
            _context.next = 22;
            break;

          case 19:
            _context.prev = 19;
            _context.t0 = _context["catch"](14);
            process.exit(1);

          case 22:
          case "end":
            return _context.stop();
        }
      }
    }, _callee, null, [[14, 19]]);
  }));

  return function dockerize() {
    return _ref.apply(this, arguments);
  };
}();

module.exports = dockerize;